/* winudp_recv.cpp 
 * MS-Windows UDP Babyphone Empfaenger (Lautsprecher)
 * Andre Adrian
 */

#include "../winudp_common.h"
#include "winudp_recv.h"

HWND hWindow;

// Audio Output (send to loudspeaker)
HANDLE WorkerThreadHandleOut;
DWORD WorkerThreadIdOut;
HWAVEOUT hWaveOut;
WAVEHDR WaveHdrOut[AUDIOBUFS];
BYTE BufferOut[AUDIOBUFS][BUFSIZE];

DWORD WINAPI UdpRecvThread (void *Arg)
{
  WSADATA wsaData;
  struct sockaddr_in local;   // UDP Empfaenger IPv4 Adress und Port Struktur
  SOCKET localfd;             // UDP Empfaenger Filedeskriptor
  int len;
  int rv;
 
  /* MS-Windows spezielle Socket Initialisierung */
  rv = WSAStartup(MAKEWORD(2, 2), &wsaData);
  winexit_if(rv != 0, hWindow, "WSAStartup failed");

  localfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP); // Socket Filedeskriptor anlegen
  winexit_if(localfd < 0, hWindow, "socket failed");

  memset(&local, 0, sizeof(local)); // local mit 0 vollschreiben
  local.sin_family = AF_INET; 
  local.sin_port = htons(BABYPHONE_PORT);
  local.sin_addr.s_addr = INADDR_ANY;

  // local Server IPv4 Adresse eintragen
  rv = bind(localfd, (struct sockaddr *) &local, sizeof(local));
  winexit_if(rv < 0, hWindow, "bind failed");

  for (;;) {                                // Endlosschleife
    struct sockaddr_in remote;  // UDP Sender IPv4 Adress und Port Struktur
    int i;
    int addr_len;
    BYTE buffer[BUFSIZE];
    
    addr_len = sizeof(remote);
    len = recvfrom(localfd, buffer, sizeof(buffer), 0, (struct sockaddr *) &remote, &addr_len);
    winexit_if(rv < 0, hWindow, "recvfrom failed");
  
    for (i = 0; i < AUDIOBUFS; ++i) {
      if (0 == WaveHdrOut[i].dwUser) {
        WaveHdrOut[i].dwUser = 1;       // Buffer belegt markieren
        CopyMemory(WaveHdrOut[i].lpData, buffer, sizeof(buffer));
        waveOutWrite(hWaveOut, WaveHdrOut + i, sizeof(WaveHdrOut[0]));
        break;
      }
    }
  }
}


BOOL CALLBACK 
DlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg) {
    case WM_INITDIALOG: {
      int i;
      int rv;
      WAVEFORMATEX waveform;
      HANDLE UdpRecvThreadHandle;
      DWORD UdpRecvThreadId;

      hWindow = hWnd;
      
      // Init Audio
      waveform.nChannels = AUDIOCHANNELS;
      waveform.wBitsPerSample = 8;
      waveform.nAvgBytesPerSec = AUDIOSPEED * waveform.nChannels * waveform.wBitsPerSample / 8;
      waveform.wFormatTag = AUDIOFORMAT;
      waveform.nSamplesPerSec = AUDIOSPEED;
      waveform.nBlockAlign = 1;
      waveform.cbSize = 0;

      rv = waveOutOpen(&hWaveOut, WAVE_MAPPER, &waveform, MAKELONG(hWnd, 0), 0, CALLBACK_WINDOW);
      winexit_if(rv != MMSYSERR_NOERROR, hWnd, "waveOutOpen failed");

      for (i = 0; i < AUDIOBUFS; ++i) {
        WaveHdrOut[i].lpData = (LPSTR) BufferOut[i];
        WaveHdrOut[i].dwBufferLength = sizeof(BufferOut[0]);
        WaveHdrOut[i].dwBytesRecorded = 0;
        WaveHdrOut[i].dwUser = 0;
        WaveHdrOut[i].dwFlags = 0;
        WaveHdrOut[i].dwLoops = 0;
        WaveHdrOut[i].lpNext = NULL;
        WaveHdrOut[i].reserved = 0;
        waveOutPrepareHeader(hWaveOut, WaveHdrOut + i, sizeof(WaveHdrOut[0]));
      }
      
      // Init Network
      UdpRecvThreadHandle = CreateThread (NULL, 0, UdpRecvThread, NULL, 0, &UdpRecvThreadId );
      winexit_if(NULL == UdpRecvThreadHandle, hWindow, "CreateThread failed");
      break;
    }

    case MM_WOM_DONE: {
      WAVEHDR *Hdr = (WAVEHDR *) lParam;

      Hdr->dwUser = 0;  // Buffer frei melden
      break;
    }

    case WM_COMMAND: {
      if (IDCANCEL == wParam) {
        EndDialog(hWnd, 0);
      }
      break;
    }
  }
  return 0;
}

int APIENTRY 
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  InitCommonControls();
  DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), HWND_DESKTOP, DlgProc);
  return 0;
}
