/* winudp_send.c
 * MS-Windows UDP Babyphone Sender (Mikrofon)
 * Andre Adrian
 */

#include "../winudp_common.h"
#include "winudp_send.h"

// Netzwerk Ausgabe
struct sockaddr_in remote;          // UDP Empfaenger IPv4 Adress und Port Struktur
SOCKET remotefd = INVALID_SOCKET;   // UDP Empfaenger Filedeskriptor

// Audio Eingabe (Empfang vom Mikrofon)
HWAVEIN hWaveIn;
WAVEHDR WaveHdrIn[AUDIOBUFS];
BYTE BufferIn[AUDIOBUFS][BUFSIZE];

BOOL CALLBACK DlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
  switch (msg) {
    case WM_INITDIALOG: {      
      WSADATA wsaData;
      int rv;
      int i;
      WAVEFORMATEX waveform;
      
      // Init Netzwerk
      rv = WSAStartup(MAKEWORD(2, 2), &wsaData);
      winexit_if(rv != 0, hWnd, "WSAStartup failed");

      // Init Audio
      waveform.nChannels = AUDIOCHANNELS;
      waveform.wBitsPerSample = 8;
      waveform.nAvgBytesPerSec = AUDIOSPEED * waveform.nChannels * waveform.wBitsPerSample / 8;
      waveform.wFormatTag = AUDIOFORMAT;
      waveform.nSamplesPerSec = AUDIOSPEED;
      waveform.nBlockAlign = 1;
      waveform.cbSize = 0;

      rv = waveInOpen(&hWaveIn, WAVE_MAPPER, &waveform, MAKELONG(hWnd, 0), 0, CALLBACK_WINDOW);
      winexit_if(rv != MMSYSERR_NOERROR, hWnd, "waveInOpen failed");

      for (i = 0; i < AUDIOBUFS; ++i) {
        WaveHdrIn[i].lpData = BufferIn[i];
        WaveHdrIn[i].dwBufferLength = sizeof(BufferIn[0]);
        WaveHdrIn[i].dwBytesRecorded = 0;
        WaveHdrIn[i].dwUser = 0;
        WaveHdrIn[i].dwFlags = 0;
        WaveHdrIn[i].dwLoops = 0;
        WaveHdrIn[i].lpNext = NULL;
        WaveHdrIn[i].reserved = 0;
        waveInPrepareHeader(hWaveIn, WaveHdrIn + i, sizeof(WaveHdrIn[0]));
        waveInAddBuffer(hWaveIn, WaveHdrIn + i, sizeof(WaveHdrIn[0]));
      }

      waveInStart(hWaveIn);   // starte Aufnahme
      break;
    }

    case MM_WIM_DATA: {
      int rv;
      WAVEHDR *Hdr = (WAVEHDR *) lParam;

      if (remotefd != INVALID_SOCKET) {
        rv = sendto(remotefd, Hdr->lpData, sizeof(BufferIn[0]), 0, 
                    (struct sockaddr *) &remote, sizeof(remote));
        if (rv < 0) {
          closesocket(remotefd);
          remotefd = INVALID_SOCKET;
          MessageBox(hWnd, "sendto failed", "WARNING", MB_OK);
        }
      }

      waveInAddBuffer(hWaveIn, Hdr, sizeof(*Hdr));
      break;
    }

    case WM_COMMAND: {
      if (ID_CONNECT == wParam) {
        struct hostent *he;
        int rv;
        char yes = '1';
        char nodename[128];

        if (remotefd != INVALID_SOCKET) {
          closesocket(remotefd);
          remotefd = INVALID_SOCKET;
        }

        // Hole den Nodename aus dem Edit Dialog
        rv = GetDlgItemText(hWnd, ID_NODENAME, nodename, sizeof(nodename));

        he = gethostbyname(nodename);  // hole UDP Empfaenger IPv4 Adresse
        winreturn_if(NULL == he, hWnd, "gethostbyname failed");

        remotefd = socket(AF_INET, SOCK_DGRAM, 0); // Socket Filedeskriptor anlegen
        winreturn_if(remotefd < 0, hWnd, "socket failed");

        // UDP Broadcast erlauben
        rv = setsockopt(remotefd, SOL_SOCKET, SO_BROADCAST, &yes, sizeof(yes));
        winreturn_if(rv < 0, hWnd, "setsockopt failed");

        memset(&remote, 0, sizeof(remote)); // remote mit 0 fuellen
        remote.sin_family = AF_INET;
        remote.sin_port = htons(BABYPHONE_PORT);
        remote.sin_addr = *((struct in_addr *) he->h_addr);
      }

      if (IDCANCEL == wParam) {
        closesocket(remotefd); 
        EndDialog(hWnd, 0);
      }
      break;
    }
  }
  return 0;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  InitCommonControls();
  DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), HWND_DESKTOP, DlgProc);
  return 0;
}
